<?php

if (!function_exists('boolval')) {
    function boolval($val)
    {
        return (bool)$val;
    }
}

class Traknpay_Tnp_PaymentController extends Mage_Core_Controller_Front_Action
{
    private $LOG_FILE_NAME = 'tnp.log';

    public function redirectAction()
    {
        try {
            $this->loadLayout();
            // Get latest order data
            $orderId = Mage::getSingleton('checkout/session')->getLastRealOrderId();
            $order = Mage::getModel('sales/order')->loadByIncrementId($orderId);

            // Set status to payment pending
            $order->setState(Mage_Sales_Model_Order::STATE_PENDING_PAYMENT, true, 'Payment initiated')->save();
            $storeId = Mage::app()->getStore()->getStoreId();
            $storeCode = Mage::app()->getStore()->getCode();
            $api_key = Mage::getStoreConfig('payment/tnp/api_key', $storeId);
            $private_salt = Mage::getStoreConfig('payment/tnp/private_salt', $storeId);
            $mode = Mage::getStoreConfig('payment/tnp/test', $storeId);
            if ($mode == '0') {
                $mode = 'LIVE';
            } else {
                $mode = 'TEST';
            }
            $name = $order->getCustomerName();
            $email = $order->getCustomerEmail();
            $phone = substr(str_replace(' ', '', $order->getBillingAddress()->getTelephone()), 0, 20);
            $currency = $order['order_currency_code'];
            $amount = $order->getBaseGrandTotal();
            $amount = $this->amount_format($amount);
            $orderid = $orderId;
            $address_line_1 = $order->getBillingAddress()->getStreet(1);
            $address_line_2 = $order->getBillingAddress()->getStreet(2);
            $city = $order->getBillingAddress()->getCity();
            $country = $order->getBillingAddress()->getCountryId();
            if ($country == 'IN') {
                $country = 'IND';
            }
            $description = 'TNPV1.4.1_MagentoV1.9.2.x: Paying invoice ' . $orderid . ' of store ' . Mage::app()->getStore()->getFrontendName() . ' using magento plugin';
            $state = $order->getBillingAddress()->getData('region');
            $zip_code = $order->getBillingAddress()->getPostcode();
            $ip = Mage::helper('core/http')->getRemoteAddr();
            $return_url = Mage::getBaseUrl(Mage_Core_Model_Store::URL_TYPE_WEB) . 'tnp/payment/response/';
            $rmTranid = time();

            $data = Array();
            $data['api_key'] = $api_key;
            $data['order_id'] = $orderid;
            $data['currency'] = $currency;
            $data['name'] = $name;
            $data['email'] = $email;
            $data['amount'] = $amount;
            $data['phone'] = $phone;
            $data['address_line_1'] = $address_line_1;
            $data['address_line_2'] = $address_line_2;
            $data['city'] = $city;
            $data['country'] = $country;
            $data['description'] = $description;
            $data['state'] = $state;
            $data['zip_code'] = $zip_code;
            $data['return_url'] = $return_url;
            $data['udf1'] = $ip;
            $data['mode'] = $mode;

            $hash_columns = [
                'address_line_1',
                'address_line_2',
                'amount',
                'api_key',
                'city',
                'country',
                'currency',
                'description',
                'email',
                'mode',
                'name',
                'order_id',
                'phone',
                'return_url',
                'state',
                'udf1',
                'udf2',
                'udf3',
                'udf4',
                'udf5',
                'zip_code',
            ];

            ksort($hash_columns);
            $hash_data = $private_salt;

            foreach ($hash_columns as $column) {
                if (isset($data[$column])) {
                    if (strlen($data[$column]) > 0) {
                        $hash_data .= '|' . $data[$column];
                    }
                }
            }
            $hash = null;
            if (strlen($hash_data) > 0) {
                $hash = strtoupper(hash("sha512", $hash_data));
            }
            $data['hash'] = $hash;

            Mage::log("Hash data sent to the serer: $hash_data", Zend_Log::INFO, $this->LOG_FILE_NAME);
            $url = 'https://biz.traknpay.in/v2/paymentrequest';

            $payment = $order->getPayment();
            $payment->setTransactionId($rmTranid); // Make it unique.
            $transaction = $payment->addTransaction(Mage_Sales_Model_Order_Payment_Transaction::TYPE_AUTH,
                null,
                false,
                '');
            $transaction->setAdditionalInformation(Mage_Sales_Model_Order_Payment_Transaction::RAW_DETAILS,
                array('Context' => 'Token payment',
                    'Amount' => $amount,
                    'Status' => 0,
                    'Url' => $url));
            $transaction->setIsTransactionClosed(false); // Close the transaction on return?
            $transaction->save();
            $order->save();

            $block = $this->getLayout()->createBlock('Mage_Core_Block_Template', 'tnp', array('template' => 'tnp/redirect.phtml'))
                ->assign(array_merge($data, array('url' => $url)));
            $this->getLayout()->getBlock('content')->append($block);
            $this->renderLayout();
        } catch (Exception $e) {
            Mage::logException($e);
            Mage::log($e, Zend_Log::ERR, $this->LOG_FILE_NAME);
            parent::_redirect('checkout/cart');
        }
    }

    // Redirect from Traknpay
    // The response action is triggered when your gateway sends back a response after processing the customer's payment
    public function responseAction()
    {
        $storeId = Mage::app()->getStore()->getStoreId();
        $storeCode = Mage::app()->getStore()->getCode();

        $response_code = $this->getRequest()->getParam('response_code');
        $response_message = $this->getRequest()->getParam('response_message');
        $transaction_id = $this->getRequest()->getParam('transaction_id');
        $amount = $this->getRequest()->getParam('amount');

        $hash_columns = $_POST;
        unset($hash_columns['hash']);

        ksort($hash_columns);
        $private_salt = Mage::getStoreConfig('payment/tnp/private_salt', $storeId);

        $hash_data = $private_salt;

        foreach ($hash_columns as $key=>$value) {
            if (strlen($value) > 0) {
                $hash_data .= '|' . $value;
            }
        }

        $hash = null;
        if (strlen($hash_data) > 0) {
            $hash = strtoupper(hash("sha512", $hash_data));
        }
        $this->loadLayout();

        $orderId = Mage::getSingleton('checkout/session')->getLastRealOrderId();
        $order = Mage::getModel('sales/order')->loadByIncrementId($orderId);

        // Set status to payment pending
        if ($hash == $this->getRequest()->getParam('hash')) {

            if ($response_code == 0) {
                // Get order details
                $order->setState(Mage_Sales_Model_Order::STATE_PROCESSING, true, 'Payment successful');
                $order->sendNewOrderEmail();
                $order->setEmailSent(true);
                $order->save();
                Mage::log("Payment ₹$amount is successful for Order Id $orderId and Transaction Id $transaction_id.", Zend_Log::INFO, $this->LOG_FILE_NAME);
                $this->_redirect('checkout/onepage/success', array('_secure' => true));
            }
            elseif ($response_message == 'Transaction Failed') {
                Mage::log("Payment unsuccessful for Order Id : $orderId", Zend_Log::ERR, $this->LOG_FILE_NAME);
                $order->cancel()->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Payment failed.')->save();
                $block = $this->getLayout()->createBlock('Mage_Core_Block_Template', 'tnp', array('template' => 'tnp/payment_failure.phtml'));
                $this->getLayout()->getBlock('content')->append($block);
                $this->renderLayout();
            }else
             {
                 $response_message=substr($response_message,strpos($response_message, ' - '));
                 $block = $this->getLayout()->createBlock('Mage_Core_Block_Template', 'tnp', array('template' => 'tnp/error.phtml'))
                     ->assign(array('error' => $response_message));
                 $this->getLayout()->getBlock('content')->append($block);
                 $this->renderLayout();
             }
        } else {
            Mage::log("Hash mis matched.", Zend_Log::ERR, $this->LOG_FILE_NAME);
            Mage::log("Hash Calculated with data sent by server is : $hash_data", Zend_Log::ERR, $this->LOG_FILE_NAME);
            Mage::log("Hash Calculated : $hash", Zend_Log::ERR, $this->LOG_FILE_NAME);
            Mage::log("Hash sent from the server is : " . $this->getRequest()->getParam('hash'), Zend_Log::ERR, $this->LOG_FILE_NAME);
            $order->cancel()->setState(Mage_Sales_Model_Order::STATE_CANCELED, true, 'Hash mis matched.')->save();
            $block = $this->getLayout()->createBlock('Mage_Core_Block_Template', 'tnp', array('template' => 'tnp/hash_failure.phtml'));
            $this->getLayout()->getBlock('content')->append($block);
            $this->renderLayout();

        }
    }

    public function amount_format($number)
    {
        $amount = round((float)$number, (int)2);
        $amount = number_format($amount, (int)2, '.', '');
        return $amount;
    }

} 